
public class FeaturePoint {
	private double x;
	private double y;
	private double z;

	/**
	 * Constructor. A FeaturePoint is composed of a (x,y,z) values
	 *
	 * @param (double x) 
	 * @param (double y) 
	 * @param (double z) 
	 * @return creates a feature point with the given values
	 */
	public FeaturePoint(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	/**
	 * 
	 * @return (double x)
	 */
	public Double getX() {
		return this.x;
	}

	/**
	 * 
	 * @return (double y)
	 */
	public Double getY() {
		return this.y;
	}

	/**
	 * 
	 * @return (double z)
	 */
	public Double getZ() {
		return this.z;
	}
	
	/**
	 * Calculates the Eulclidean distance between this and another FeaturePoint.
	 * D = SQUARE_ROOT((x1 - x2)^2 + (y1 - y2)^2 + (z1 - z2)^2)
	 * 
	 * @param (FeaturePoint f1) featurepoint to get distance to
	 * @return (double) the 3D distance between the 2 points
	 */
	public Double distance(FeaturePoint f1) {
		double x = f1.getX();
		double y = f1.getY();
		double z = f1.getZ();
		double xyzDistance = ((x - this.x) * (x - this.x)) + ((y - this.y) * (y - this.y)) + ((z - this.z) * (z - this.z));
		xyzDistance = Math.sqrt(xyzDistance);
		return xyzDistance;
	}
}
