import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

public class FeatureVector {

	private ArrayList<FeaturePoint> featureVector; // composed of 20 FeaturePoints that xbox kinect detects
	private static final String[] featurePoints = {"HipCenter", "Spine", "ShoulderCenter", "Head", "ShoulderLeft", "ElbowLeft",
		"WristLeft", "HandLeft", "ShoulderRight", "ElbowRight", "WristRight", "HandRight", "HipLeft", "KneeLeft", "AnkleLeft", 
		"FootLeft", "HipRight", "KneeRight", "AnkleRight", "FootRight"};

	/**
	 * Constructor. A FeaturePoint is composed of a (x,y,z) values
	 *
	 * @param (double x) 
	 * @param (double y) 
	 * @param (double z) 
	 * @return creates a feature point with the given values
	 */
	public FeatureVector(String XYZfilename) throws FileNotFoundException{
		Scanner in;
		this.featureVector = new ArrayList<FeaturePoint>();
		String s1;
		String[] splitLine;
		in = new Scanner(new File(XYZfilename));
		while(in.hasNext()) {
			s1 = in.nextLine();
			splitLine = s1.split(" ");
			FeaturePoint f1;
			f1 = new FeaturePoint(Double.parseDouble(splitLine[0]), Double.parseDouble(splitLine[1]), (double)(Double.parseDouble(splitLine[2])));
			this.featureVector.add(f1);
		}
	}

	/**
	 * returns the feature point with the given name
	 *
	 * @param (String name) 
	 * @return FeaturePoint with given name
	 */
	public FeaturePoint get(String name) {
		for(int i = 0; i < FeatureVector.featurePoints.length; i++) {
			String s1 = FeatureVector.featurePoints[i];
			if(s1.equals(name)) {
				return this.featureVector.get(i);
			}
		}
		return null;
	}

	/**
	 * returns the number of feature points in the feature vector
	 *
	 * @param none
	 * @return number of feature points
	 */
	public int size() {
		return this.featureVector.size();
	}

	/**
	 * returns the feature point with that is at the specified index
	 *
	 * @param (int index) index to get featurepoint 
	 * @return FeaturePoint with given index
	 */
	public FeaturePoint get(int index) {
		return this.featureVector.get(index);
	}

	/**
	 * returns ArrayList with all the feature points in it
	 *
	 * @param none 
	 * @return (ArrayList<FeaturePoint>)
	 */
	public ArrayList<FeaturePoint> getFeaturePoints() {
		return this.featureVector;
	}

}
