import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.*;

/*
 * 
 * contatins some unused methods that were developed but then not needed. These Java file were used more in testing, 
 * as the main application with GUI uses .NET and VB.
 */
public class Password {

	private ArrayList<FeatureVector> trainingSet;
	private ArrayList<Double> thresholds;
	private ArrayList<Double> xThresholds;
	private ArrayList<Double> yThresholds;
	private ArrayList<Double> zThresholds;
	// distance from head to hip center, we will store the average distance calculated from the entire training set. Somewhat reliable, but not fool proof
	private double usersHeight; 
	private double usersHeightThreshold;
	private ArrayList<FeaturePoint> password; // will be the average value of points from training set
	private static final String[] featurePoints = {"HipCenter", "Spine", "ShoulderCenter", "Head", "ShoulderLeft", "ElbowLeft",
		"WristLeft", "HandLeft", "ShoulderRight", "ElbowRight", "WristRight", "HandRight", "HipLeft", "KneeLeft", "AnkleLeft", 
		"FootLeft", "HipRight", "KneeRight", "AnkleRight", "FootRight"};
	private final double thresholdConstant;
	private final double xthresholdConstant;
	private final double ythresholdConstant;
	private final double zthresholdConstant;
	private FeatureVector mostRecentLogin;

	/**
	 * Constructor. Creates a password given a list of FeatureVectors
	 *
	 * @param (ArrayList<FeatureVector> train) the training set for the password
	 * @return Password Object
	 */
	public Password(ArrayList<FeatureVector> train, double constant) {
		this.trainingSet = train; // save the training set
		this.thresholdConstant = constant;
		this.password = new ArrayList<FeaturePoint>();
		this.setPassword();
		this.thresholds = new ArrayList<Double>();
		this.usersHeight = 0;
		this.usersHeightThreshold = 0;
		this.calculateThresholds();
		this.setBiometrics();
		this.zthresholdConstant = 0;
		this.ythresholdConstant = 0;
		this.xthresholdConstant = 0;
	}
	
	public Password(ArrayList<FeatureVector> train, double xConstant, double yConstant, double zConstant) {
		this.trainingSet = train; // save the training set
		this.thresholdConstant = 0;
		this.xthresholdConstant = xConstant;
		this.ythresholdConstant = yConstant;
		this.zthresholdConstant = zConstant;
		this.password = new ArrayList<FeaturePoint>();
		this.setPassword();
		this.thresholds = new ArrayList<Double>();
		this.xThresholds = new ArrayList<Double>();
		this.yThresholds = new ArrayList<Double>();
		this.zThresholds = new ArrayList<Double>();
		this.usersHeight = 0;
		this.usersHeightThreshold = 0;
		this.calculateIndividualXYZThresholds();
		this.setBiometrics();
	}

	/**
	 * Creates the password using the training set. The password consists of the average value for each feature point.
	 * The password class variable is set after this method executes.
	 *
	 * @param none 
	 * @return none
	 */
	private void setPassword() {
		FeaturePoint p1;
		double x = 0;
		double y = 0;
		double z = 0;
		for(int i = 0; i < this.trainingSet.get(0).size(); i++) {
			x = 0; y = 0; z = 0;
			for(int a = 0; a < this.trainingSet.size(); a++) {
				p1 = this.trainingSet.get(a).get(i);
				x += p1.getX();
				y += p1.getY();
				z += p1.getZ();
			}
			x = x / this.trainingSet.size();
			y = y / this.trainingSet.size();
			z = z / this.trainingSet.size();
			FeaturePoint average = new FeaturePoint(x, y, z);
			this.password.add(average);
		}
	}

	/**
	 * figures out the percentage that a login attempt matched the password
	 *
	 * @param (int numFailedToMatch) the number of feature points that failed to match the password 
	 * @return (Double) the percent that the login attemp matched the password
	 */
	public Double percentMatching(int numFailedToMatch) {
		double m = (double)(this.password.size() - numFailedToMatch) / this.password.size();
		return m * 100;
	}

	/**
	 * This is a login attempt. The object is past some other FeatureVector, which it compares to the values
	 * that it has stored as the password. The users height as well as each feature point is compared.
	 *
	 * @param (FeatureVector f1) some other feature vector, presumably an attemped login 
	 * @return (int) the number of feature points that failed to match the password
	 */
	public int login(FeatureVector f1) {
		this.mostRecentLogin = f1; // save this login
		// get the height of person from f1 and compare to the height of the known user
		double height = f1.get("Head").distance(f1.get("HipCenter")); 
		if(this.distance(height, this.usersHeight) > this.usersHeightThreshold) {
			System.out.println("height does not match");
		}
		else {
			System.out.println("height matches");
		} 
		ArrayList<FeaturePoint> fp1 = f1.getFeaturePoints();
		ArrayList<FeaturePoint> passwordFeaturePoints = this.password;
		int count = 0;
		for(int i = 0; i < fp1.size(); i++) {
			if(fp1.get(i).distance(passwordFeaturePoints.get(i)) > this.thresholds.get(i)) {
				System.out.println("failed at point: " + featurePoints[i]);
				System.out.println("Distance: " + fp1.get(i).distance(passwordFeaturePoints.get(i)));
				count++;
			}
		}
		return count;
	}
	
	/**
	 * This is a login attempt. The object is past some other FeatureVector, which it compares to the values
	 * that it has stored as the password. The users height as well as each feature point is compared. It is different
	 * from above login in that it looks for differences in X,Y,and Z seperately as opposed to together. 
	 *
	 * @param (FeatureVector f1) some other feature vector, presumably an attempted login 
	 * @return (int) the number of feature points that failed to match the password
	 */
	public int[] login(FeatureVector f1, int a) {
		this.mostRecentLogin = f1; // save this login
		// get the height of person from f1 and compare to the height of the known user
		double height = f1.get("Head").distance(f1.get("HipCenter")); 
		if(this.distance(height, this.usersHeight) > this.usersHeightThreshold) {
			System.out.println("height does not match");
		}
		else {
			System.out.println("height matches");
		} 
		ArrayList<FeaturePoint> fp1 = f1.getFeaturePoints();
		ArrayList<FeaturePoint> passwordFeaturePoints = this.password;
		int xCount = 0;
		int yCount = 0;
		int zCount = 0;
		int[] countReturn = new int[3];
		for(int i = 0; i < fp1.size(); i++) {
			if(this.distance(fp1.get(i).getX(), passwordFeaturePoints.get(i).getX()) > this.xThresholds.get(i)) {
				System.out.println("failed in X component at point: " + featurePoints[i]);
				System.out.println("Distance: " + this.distance(fp1.get(i).getX(), passwordFeaturePoints.get(i).getX()));
				xCount++;
			}
			if(this.distance(fp1.get(i).getY(), passwordFeaturePoints.get(i).getY()) > this.yThresholds.get(i)) {
				System.out.println("failed in Y component at point: " + featurePoints[i]);
				System.out.println("Distance: " + this.distance(fp1.get(i).getY(), passwordFeaturePoints.get(i).getY()));
				yCount++;
			}
			if(this.distance(fp1.get(i).getZ(), passwordFeaturePoints.get(i).getZ()) > this.zThresholds.get(i)) {
				System.out.println("failed in Z component at point: " + featurePoints[i]);
				System.out.println("Distance: " + this.distance(fp1.get(i).getZ(), passwordFeaturePoints.get(i).getZ()));
				zCount++;
			}
		}
		countReturn[0] = xCount;
		countReturn[1] = yCount;
		countReturn[2] = zCount;
		return countReturn;
	}


	/**
	 * calculates acceptable thresholds for each FeaturePoint using the training set. The acceptable threshold is the max distance
	 * between the same feature point within the training set. Essentially we say that an acceptable threshold is the worst job
	 * the user was able to in re-position their feature point in the same place. Note that this does not set the users height
	 * or the users height threshold.
	 *
	 * @param none
	 * @return (void) note that this initializes the ArrayList<Double> thresholds
	 */
	private void calculateThresholds() {
		double maxDistance = -1;
		double tempDistance = -1;
		FeaturePoint p1;
		FeaturePoint p2;
		for(int i = 0; i < this.trainingSet.get(0).size(); i++) {
			p1 = this.trainingSet.get(0).get(i); // go through all points of 1st feature vector and compare with all other corresponding points in other FeatureVector
			for(int x = 1; x < this.trainingSet.size(); x++) {
				p2 = this.trainingSet.get(x).get(i);
				tempDistance = p1.distance(p2);
				if(tempDistance > maxDistance) {
					maxDistance = tempDistance;
				}
			}
			thresholds.add(maxDistance + thresholdConstant);
			maxDistance = 0;
		}
	}
	
	/**
	 * calculates acceptable thresholds for each FeaturePoint's X, Y, and Z using the training set. The acceptable threshold is the max distance
	 * between the same feature point within the training set. Essentially we say that an acceptable threshold is the worst job
	 * the user was able to in re-position their feature point in the same place. Note that this does not set the users height
	 * or the users height threshold. 
	 *
	 * @param none
	 * @return (void) note that this initializes the ArrayList<Double> thresholds
	 */
	private void calculateIndividualXYZThresholds() {
		double x1 = 0;
		double x2 = 0;
		double y1 = 0;
		double y2 = 0;
		double z1 = 0;
		double z2 = 0;
		double maxXDistance = -1;
		double tempXDistance = -1;
		double maxYDistance = -1;
		double tempYDistance = -1;
		double maxZDistance = -1;
		double tempZDistance = -1;
		FeaturePoint p1;
		FeaturePoint p2;
		for(int i = 0; i < this.trainingSet.get(0).size(); i++) {
			p1 = this.trainingSet.get(0).get(i); // go through all points of 1st feature vector and compare with all other corresponding points in other FeatureVector
			x1 = p1.getX();
			y1 = p1.getY();
			z1 = p1.getZ();
			for(int x = 1; x < this.trainingSet.size(); x++) {
				p2 = this.trainingSet.get(x).get(i);
				x2 = p2.getX();
				y2 = p2.getY();
				z2 = p2.getZ();
				tempXDistance = this.distance(x1, x2);
				tempYDistance = this.distance(y1, y2);
				tempZDistance = this.distance(z1, z2);
				if(tempXDistance > maxXDistance) {
					maxXDistance = tempXDistance;
				}
				if(tempYDistance > maxYDistance) {
					maxYDistance = tempYDistance;
				}
				if(tempZDistance > maxZDistance) {
					maxZDistance = tempZDistance;
				}
			}
			this.xThresholds.add(maxXDistance + this.xthresholdConstant);
			this.yThresholds.add(maxYDistance + this.ythresholdConstant);
			this.zThresholds.add(maxZDistance + this.zthresholdConstant);
			maxXDistance = 0;
			maxYDistance = 0;
			maxZDistance = 0;
		}
	}

	/**
	 * Each feature vector has a list of FeaturePoints that are meant to be biometrics => identifying characteristics. As of now we will be using
	 * the persons height (distance from head to hip center). This method calculates the average distance from head to hip center based on the training
	 * set as well as calculates an acceptable threshold for the users height. Again, we use the max distance between the users height that occured
	 * in the training set as the threshold. Note that this sets the users height as well as the users height threshold
	 * @param none
	 * @return (void) note that this initializes the users height and height threshold
	 */
	private void setBiometrics() {
		ArrayList<Double> heights = new ArrayList<Double>();
		for(int x = 0; x < this.trainingSet.size(); x++) {
			heights.add((this.trainingSet.get(x).get("Head").distance(this.trainingSet.get(x).get("HipCenter"))));
			}
		// calculate the users average height using points from the training set
		for(int i = 0; i < heights.size(); i++) {
			this.usersHeight = this.usersHeight + heights.get(i);
		}
		this.usersHeight = this.usersHeight / heights.size();

		double height1 = 0;
		double height2 = 0;
		double tempHeightDistance = 0;
		double maxHeightDistance = 0;
		// find threshold for height
		for(int i = 0; i < heights.size(); i++) {
			height1 = heights.get(i);
			for(int x = i + 1; x < heights.size(); x++){
				height2 = heights.get(x);
				tempHeightDistance = this.distance(height1, height2);
				if(tempHeightDistance > maxHeightDistance) {
					maxHeightDistance = tempHeightDistance;
				}
			}
		}
		this.usersHeightThreshold = maxHeightDistance + thresholdConstant;
	}

	/**
	 * outputs useful information to a text file
	 *
	 * @param (String filename) 
	 * @return void
	 */
	public void writeToTextFile(String filename) {
		try {
			PrintWriter out = new PrintWriter(filename);
			out.println("Thresholds for all feature points: ");
			for(int i = 0; i < this.thresholds.size(); i++) {
				out.println(featurePoints[i] + ": " + this.thresholds.get(i));
			}
			out.println();
			out.println("Threshold for users height: " + this.usersHeightThreshold);
			out.println("Users height: " + this.usersHeight);
			out.println();
			out.println("Users password (average of the training set): ");
			for(int i = 0; i < this.password.size(); i++) {
				FeaturePoint p1 = this.password.get(i);
				out.println(p1.getX() + " " + p1.getY() + " " + p1.getZ());
			}
			out.close();
		} catch(FileNotFoundException e) {
			System.out.println(e.getMessage());
			return;
		}
		return;
	}

	/**
	 * outputs useful information to the console
	 *
	 * @param none
	 * @return void
	 */
	public void output(){
		System.out.println("Thresholds for all feature points: ");
		for(int i = 0; i < this.thresholds.size(); i++) {
			System.out.println(featurePoints[i] + ": " + this.thresholds.get(i));
		}
		System.out.println();
		System.out.println("Threshold for users height: " + this.usersHeightThreshold);
		System.out.println("Users height: " + this.usersHeight);
		double height = this.mostRecentLogin.get("Head").distance(this.mostRecentLogin.get("HipCenter"));
		System.out.println("Height of previous person to login: " + height);
	}
	
	/**
	 * outputs useful information to the console
	 *
	 * @param none
	 * @return void
	 */
	public void output(int index){
		System.out.println("Thresholds for all feature points: ");
		for(int i = 0; i < this.xThresholds.size(); i++) {
			System.out.println(featurePoints[i] + " X: " + this.xThresholds.get(i) + " Y: " +  this.yThresholds.get(i) + " Z: " +  this.zThresholds.get(i));
		}
		System.out.println();
		System.out.println("Threshold for users height: " + this.usersHeightThreshold);
		System.out.println("Users height: " + this.usersHeight);
		double height = this.mostRecentLogin.get("Head").distance(this.mostRecentLogin.get("HipCenter"));
		System.out.println("Height of previous person to login: " + height);
	}

	/**
	 * Calculates 1 dimensional distance between 2 numbers
	 *
	 * @param (Double x1) 
	 * @param (Double x2)
	 * @return (Double) the distance between x1 and x2
	 */
	private Double distance(Double x1, Double x2) {
		return (Math.abs(x1 - x2));
	}
}
