Imports System

''' <summary>
''' The class that will hold the feature point values
''' </summary>
''' <remarks></remarks>
Public Class FeaturePoint
	Private x As Double
	Private y As Double
	Private z As Double

    ''' <summary>
    ''' Constructor. A FeaturePoint is composed of a (x,y,z) values
    ''' </summary>
    ''' <param name="x"> </param>
    ''' <param name="y"> </param>
    ''' <param name="z"> </param>
    ''' <remarks></remarks>
	Public Sub New(ByVal x As Double, ByVal y As Double, ByVal z As Double)
		Me.x = x
		Me.y = y
		Me.z = z
	End Sub

    ''' 
    ''' <returns> (double x) </returns>
    Public Overridable ReadOnly Property xVal As Double
        Get
            Return Me.x
        End Get
    End Property

    ''' 
    ''' <returns> (double y) </returns>
    Public Overridable ReadOnly Property yVal As Double
        Get
            Return Me.y
        End Get
    End Property

    '''
    ''' <returns> (double z) </returns>
    Public Overridable ReadOnly Property zVal As Double
        Get
            Return Me.z
        End Get
    End Property

    ''' <summary>
    ''' Calculates the Eulclidean distance between this and another FeaturePoint.
    ''' D = SQUARE_ROOT((x1 - x2)^2 + (y1 - y2)^2 + (z1 - z2)^2)
    ''' </summary>
    ''' <param name="f1"> featurepoint to get distance to </param>
    ''' <returns> (double) the 3D distance between the 2 points </returns>
    Public Overridable Function distance(ByVal f1 As FeaturePoint) As Double
        Dim x_Renamed As Double = f1.x
        Dim y_Renamed As Double = f1.y
        Dim z_Renamed As Double = f1.z
        Dim xyzDistance As Double = ((x_Renamed - Me.x) * (x_Renamed - Me.x)) + ((y_Renamed - Me.y) * (y_Renamed - Me.y)) + ((z_Renamed - Me.z) * (z_Renamed - Me.z))
        xyzDistance = Math.Sqrt(xyzDistance)
        Return xyzDistance
    End Function


End Class
