Imports System
Imports System.Collections.Generic
Imports System.IO

''' <summary>
''' The class that will hold the feature vector
''' </summary>
''' <remarks></remarks>
Public Class FeatureVector

    Private myFeatureVector As List(Of FeaturePoint) ' composed of 20 FeaturePoints that xbox kinect detects
    Private Shared ReadOnly featurePoints() As String = {"HipCenter", "Spine", "ShoulderCenter", "Head", "ShoulderLeft", "ElbowLeft", "WristLeft", "HandLeft", "ShoulderRight", "ElbowRight", "WristRight", "HandRight", "HipLeft", "KneeLeft", "AnkleLeft", "FootLeft", "HipRight", "KneeRight", "AnkleRight", "FootRight"}

    ''' <summary>
    ''' Constructor. A FeaturePoint is composed of a (x,y,z) values
    ''' </summary>
    ''' <param name="XYZfilename"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal XYZfilename As String)
        Dim myIn As New StreamReader(XYZfilename)
        Me.myFeatureVector = New List(Of FeaturePoint)()
        Dim s1 As String
        Dim splitLine() As String
        Do
            s1 = myIn.ReadLine
            If Not (s1 Is Nothing) Then
                splitLine = StringHelperClass.StringSplit(s1, " ", True)
                Dim f1 As FeaturePoint
                f1 = New FeaturePoint(Convert.ToDouble(splitLine(0)), Convert.ToDouble(splitLine(1)), Convert.ToDouble(splitLine(2)))
                Me.myFeatureVector.Add(f1)
            End If
        Loop Until s1 Is Nothing
    End Sub

    ''' <summary>
    ''' returns the feature point with the given name
    ''' </summary>
    ''' <param name="name"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function getFeaturePoint(ByVal name As String) As FeaturePoint
        For i As Integer = 0 To FeatureVector.featurePoints.Length - 1
            Dim s1 As String = FeatureVector.featurePoints(i)
            If s1.Equals(name) Then
                Return Me.myFeatureVector(i)
            End If
        Next i
        Return Nothing
    End Function

    ''' <summary>
    ''' returns the number of feature points in the feature vector
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function size() As Integer
        Return Me.myFeatureVector.Count
    End Function

    ''' <summary>
    ''' returns the feature point with that is at the specified index
    ''' </summary>
    ''' <param name="index"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function getFeaturePoint(ByVal index As Integer) As FeaturePoint
        Return Me.myFeatureVector(index)
    End Function

    ''' <summary>
    ''' returns ArrayList with all the feature points in it
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable ReadOnly Property featurePointsList As List(Of FeaturePoint)
        Get
            Return Me.myFeatureVector
        End Get
    End Property

End Class
