Imports Microsoft.Kinect
Imports System.IO
Imports System
Imports System.Globalization

''' <summary>
''' The main application window for Kinect Capture
''' </summary>
''' <remarks></remarks>
Class MainWindow

    Private frameTaken As Integer = 0
    Private mySensor As KinectSensor
    Private colorWriteableBitmap As WriteableBitmap
    Private colorPixelData As Byte()
    Private myEncoder(1) As PngBitmapEncoder
    Private myColorFrame As BitmapFrame
    'Private curTime As Date = DateTime.UtcNow
    Private myFileOut As FileStream
    Private convertedBodyPoints(19) As MyPosition
    Private WithEvents myForm1 As multipurpose
    Private WithEvents myFormSaveTofolder As String
    Private WithEvents myForm2 As String
    Private quitLiveModeStatus As Boolean = True
    Private dirHistory As String() = {"", ""}
    Private mydriver As New driver
    Private frameCounter As Integer = 0

    ''' <summary>
    ''' Things to do when main app closes
    ''' - Stop sensor
    ''' - Remove temp folder and file
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainWindow_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        Try
            Me.mySensor.Stop()
        Catch ex As Exception
            MessageBox.Show("Sensor already stopped")
        End Try
        If Directory.Exists("./Attempt_Temp/") Then
            For Each foundFile As String In My.Computer.FileSystem.GetFiles("./Attempt_Temp/", Microsoft.VisualBasic.FileIO.SearchOption.SearchAllSubDirectories, "*.*")
                My.Computer.FileSystem.DeleteFile(foundFile, FileIO.UIOption.OnlyErrorDialogs, Microsoft.VisualBasic.FileIO.RecycleOption.DeletePermanently)
            Next
            Directory.Delete("./Attempt_Temp/")
        End If

    End Sub

    ''' <summary>
    ''' Things to do when the main app loads
    ''' - check for kinect existence
    ''' - start the kinect sensor
    ''' - enable all needed stream for skeleton
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainWindow_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        If Microsoft.Kinect.KinectSensor.KinectSensors.Count <= 0 Then
            MessageBox.Show("Kinect not Found")
            Me.Close()
        End If

        Try
            Me.mySensor = KinectSensor.KinectSensors(0)
        Catch ex As Exception
            MessageBox.Show("Error loading sensor")
            Me.Close()
        End Try
        Try
            Me.mySensor.Start()
            Me.mySensor.ElevationAngle = 0
        Catch ex As Exception
            MessageBox.Show("Error starting sensor")
            Me.Close()
        End Try

        Try
            Me.mySensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Me.mySensor.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30)
            Me.mySensor.SkeletonStream.Enable(New TransformSmoothParameters)
            Me.colorWriteableBitmap = New WriteableBitmap(Me.mySensor.ColorStream.FrameWidth, Me.mySensor.ColorStream.FrameHeight, 96.0, 96.0, PixelFormats.Bgr32, Nothing)
            Me.colorPixelData = New Byte(Me.mySensor.ColorStream.FramePixelDataLength - 1) {}
        Catch ex As Exception
            Me.mySensor.Stop()
            MessageBox.Show("Error 1")
            Me.Close()
        End Try

        AddHandler Me.mySensor.AllFramesReady, AddressOf subAllFramesReady
        Me.Image1.Source = Me.colorWriteableBitmap
        Me.labelFrameTaken.Content = CStr(Me.frameTaken)
    End Sub


    ''' <summary>
    ''' Things to do when all frames needed for skeleton stream is ready
    ''' - get the skeleton data from skeleton stream
    ''' - convert skeleton data into feature vector
    ''' - update image on screen
    ''' - writes a temp file (the feature vectors of current frame) if it is in the live mode
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub subAllFramesReady(ByVal sender As Object, ByVal e As AllFramesReadyEventArgs)
        Dim colorFrame As ColorImageFrame = e.OpenColorImageFrame
        Dim depthFrame As DepthImageFrame = e.OpenDepthImageFrame
        Dim skeletonFrame As SkeletonFrame = e.OpenSkeletonFrame
        If colorFrame Is Nothing Or depthFrame Is Nothing Or skeletonFrame Is Nothing Then
            If depthFrame Is Nothing Then
                labelMisc.Foreground = Brushes.Red
                labelMisc.Content = "Null depth frame"
            Else
                labelMisc.Foreground = Brushes.Black
                labelMisc.Content = ""
            End If
            Return
        End If
        labelMisc.Foreground = Brushes.Black
        labelMisc.Content = ""

        'Get the skeleton data
        Dim skeletonData(skeletonFrame.SkeletonArrayLength - 1) As Skeleton
        skeletonFrame.CopySkeletonDataTo(skeletonData)
        Dim counter As Integer = 0
        For Each skel In skeletonData
            If skel.TrackingState = SkeletonTrackingState.PositionOnly Or skel.TrackingState = SkeletonTrackingState.Tracked Then
                counter = counter + 1
                ' Using the "forbidden" goto instruction
                GoTo endLoop
            Else
                counter = counter + 1
            End If
        Next
endLoop:
        Dim skeletonDetected As Boolean = False
        If counter < skeletonData.Length Then
            Dim mySkeleton As Skeleton = skeletonData(counter - 1)
            Dim convertedIndex As Integer = 0
            Dim enumValues As Array = [Enum].GetValues(GetType(JointType))
            For Each jT As JointType In enumValues
                Dim mySkeletonPoint = mySkeleton.Joints(jT).Position
                Me.convertedBodyPoints(convertedIndex) = New MyPosition(CInt(mySkeletonPoint.X * 1000), CInt(mySkeletonPoint.Y * 1000), CInt(mySkeletonPoint.Z * 1000)) 'Me.mySensor.CoordinateMapper.MapSkeletonPointToDepthPoint(mySkeletonPoint, DepthImageFormat.Resolution640x480Fps30)
                convertedIndex = convertedIndex + 1
            Next

            skeletonDetected = True
            labelSkeletonStatus.Foreground = Brushes.Green
            labelSkeletonStatus.Content = "Skeleton detected"

        Else
            labelSkeletonStatus.Foreground = Brushes.Red
            labelSkeletonStatus.Content = "Skeleton not detected"
            labelTemp.Content = ""
        End If


        'Display color image on screen
        colorFrame.CopyPixelDataTo(Me.colorPixelData)
        Dim indexColor As Integer = 0
        Dim indexDepth As Integer = 0
        If skeletonDetected Then
            Me.labelTemp.Content = "X: " + CStr(Me.convertedBodyPoints(1).getXInmm) + " Y: " + CStr(Me.convertedBodyPoints(1).getYInmm) + " Z: " + CStr(Me.convertedBodyPoints(1).getZInmm)
            If Me.butLiveMode.Content.Equals("Live Mode") Then
                Me.Button1.IsEnabled = True
            Else
                Me.Button1.IsEnabled = False
            End If


            Me.butLiveMode.IsEnabled = True
        Else
            If Me.quitLiveModeStatus = False Then
                Me.stopLiveMode()
            End If
            Me.Button1.IsEnabled = False
            Me.butLiveMode.IsEnabled = False
        End If

        Me.colorWriteableBitmap.WritePixels(New Int32Rect(0, 0, Me.colorWriteableBitmap.PixelWidth, Me.colorWriteableBitmap.PixelHeight), Me.colorPixelData, Me.colorWriteableBitmap.PixelWidth * Len(New Integer), 0)

        'Writes temporary attempt file
        If Me.quitLiveModeStatus = False Then
            Try
                If Not Directory.Exists("Attempt_Temp/") Then
                    Directory.CreateDirectory("Attempt_Temp/")
                End If
            Catch ex As Exception
                MessageBox.Show(ex.Message)
                Me.Close()
            End Try
            Me.frameCounter = Me.frameCounter + 1
            If CDbl(Me.frameCounter / 30) = 1 Then
                Dim StreamWriter As StreamWriter
                StreamWriter = New StreamWriter("Attempt_Temp/" + "attempt.txt")
                For Each value As MyPosition In Me.convertedBodyPoints
                    StreamWriter.WriteLine(CStr(value.getXInmm) + " " + CStr(value.getYInmm) + " " + CStr(value.getZInmm))
                Next
                StreamWriter.Close()
                Me.frameCounter = 0
                GC.Collect()
                Me.liveModeReady()
            End If
            Me.labelDebug.Content = CStr(Me.frameCounter)
        Else
            Me.labelDebug.Content = CStr(Me.frameCounter)
            GC.Collect()
        End If
    End Sub

    ''' <summary>
    ''' Things to do when Snap button is clicked
    ''' - prepare directory to save image and feature vector
    ''' - call snapReady() when directory is prepared
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles Button1.Click
        Me.myColorFrame = BitmapFrame.Create(Me.colorWriteableBitmap)
        Me.toggleAllButton()
        If Me.myColorFrame Is Nothing Then
            MessageBox.Show("NULL")
            Return
        End If
        Me.myEncoder(0) = New PngBitmapEncoder()
        Me.myEncoder(1) = New PngBitmapEncoder()
        Me.myEncoder(0).Frames.Add(Me.myColorFrame)
        If Me.frameTaken = 0 Then
            Dim folderBrowser As New System.Windows.Forms.FolderBrowserDialog
            folderBrowser.SelectedPath = Me.dirHistory(0)
            Dim result As System.Windows.Forms.DialogResult = folderBrowser.ShowDialog()
            If result = Forms.DialogResult.Cancel Then
                Return
            End If
            Me.myFormSaveTofolder = folderBrowser.SelectedPath + "\"
            Me.snapReady()

        Else
            Me.snapReady()
        End If

    End Sub


    ''' <summary>
    ''' Things to do when snapReady() is called
    ''' - save the color image and feature vector to the folder prepared earlier
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub snapReady()
        Dim folderName As String = Me.myFormSaveTofolder
        If folderName.Equals("") Then
            folderName = "./"
        End If
        Me.dirHistory(0) = folderName
        Try
            If Not Directory.Exists(folderName) Then
                Directory.CreateDirectory(folderName)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Me.Close()
        End Try
        Try
            Me.myFileOut = New FileStream(folderName + "colorStream" + CStr(Me.frameTaken + 1) + ".png", FileMode.Create)
            Me.myEncoder(0).Save(Me.myFileOut)
            Me.myFileOut.Close()
        Catch ex As Exception
            MessageBox.Show("Error saving image")
            Me.Close()
        End Try
        Try
            Dim streamWriter As New StreamWriter(folderName + "fv" + CStr(Me.frameTaken + 1) + ".txt")
            For Each value As MyPosition In Me.convertedBodyPoints
                streamWriter.WriteLine(CStr(value.getXInmm) + " " + CStr(value.getYInmm) + " " + CStr(value.getZInmm))
            Next
            streamWriter.Close()
        Catch ex As Exception
            MessageBox.Show("Error saving feature vectors")
            Me.Close()
        End Try
        Me.toggleAllButton()
        Me.frameTaken = Me.frameTaken + 1
        Me.labelFrameTaken.Content = CStr(Me.frameTaken)
    End Sub

    ''' <summary>
    ''' Things to do when manual mode is clicked
    ''' - Prepares the directory for the password and attempt comparison
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub butDriver_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles butDriver.Click
        Me.myForm1 = New multipurpose
        Me.myForm1.formTitle("Directory")
        Me.myForm1.setMode(2)
        Me.myForm1.setLabelItem("Please enter the directory to the feature vectors: ")
        Me.myForm1.setTextBoxVal(Me.dirHistory(0))
        Me.myForm1.setTextBoxVal2(Me.dirHistory(1))
        Me.myForm1.setLabelItem2("Please enter the directory to the attempt file: ")
        Me.myForm1.toggleSecondLevel()
        Me.myForm1.Show()
    End Sub

    ''' <summary>
    ''' Things to do once the directory information are ready
    ''' - call the mainDriver method that will handle the comparison
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub driverReady() Handles myForm1.formReady
        Me.myForm1.Hide()
        Dim myDriver As New driver
        Dim temp1 As String = Me.myForm1.getTextBoxVal
        Dim temp2 As String = Me.myForm1.getTextBoxVal2
        If temp1.Equals("") Then
            temp1 = "./"
        End If
        If temp2.Equals("") Then
            temp2 = "./"
        End If
        Me.dirHistory(0) = temp1
        Me.dirHistory(1) = temp2
        myDriver.setNumFiles(CInt(Me.TextBox1.Text))
        myDriver.mainDriver(temp1, temp2)
    End Sub

    ''' <summary>
    ''' Reset the no. of frame taken when the label is double-clicked
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub labelFrameTaken_MouseDoubleClick(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles labelFrameTaken.MouseDoubleClick
        Me.frameTaken = 0
        Me.labelFrameTaken.Content = CStr(Me.frameTaken)
    End Sub

    ''' <summary>
    ''' Things to do when live mode is clicked
    ''' - if it is live mode then gets the directory of feature vectors that the user wanted to compare with
    ''' - if it is quit live mode then stop the live mode
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub butLiveMode_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles butLiveMode.Click
        If quitLiveModeStatus = True Then
            Dim folderBrowser As New System.Windows.Forms.FolderBrowserDialog
            folderBrowser.SelectedPath = Me.dirHistory(0)
            Me.TextBox1.IsReadOnly = True
            Dim result As System.Windows.Forms.DialogResult = folderBrowser.ShowDialog()

            If result = Forms.DialogResult.Cancel Then
                Return
            End If
            Me.myForm2 = folderBrowser.SelectedPath + "\"

            Me.liveModeReady()
        Else
            Me.stopLiveMode()
        End If

    End Sub

    ''' <summary>
    ''' Starts the live mode comparison by calling mainDriver()
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub liveModeReady()
        Dim temp As String
        If Me.quitLiveModeStatus = True Then
            Me.butLiveMode.Content = "Quit Live Mode"
            temp = Me.myForm2
            Me.dirHistory(0) = temp
        Else
            temp = Me.dirHistory(0)
        End If

        Me.quitLiveModeStatus = False

        If temp.Equals("") Then
            temp = "./"
        End If
        Me.mydriver.onLiveMode()
        Me.mydriver.setNumFiles(CInt(Me.TextBox1.Text))
        Me.mydriver.mainDriver(temp, "Attempt_Temp/")
    End Sub

    ''' <summary>
    ''' Increase the camera angle by 5 degree
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub butIncreaseElevation_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles butIncreaseElevation.Click
        If Me.mySensor.ElevationAngle + 5 > Me.mySensor.MaxElevationAngle Then
            Me.mySensor.ElevationAngle = Me.mySensor.MaxElevationAngle
        Else
            Me.mySensor.ElevationAngle = Me.mySensor.ElevationAngle + 5
        End If
    End Sub

    ''' <summary>
    ''' Decrese the camera angle by 5 degree
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub butDecreaseElevation_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles butDecreaseElevation.Click
        If Me.mySensor.ElevationAngle - 5 < Me.mySensor.MinElevationAngle Then
            Me.mySensor.ElevationAngle = Me.mySensor.MinElevationAngle
        Else
            Me.mySensor.ElevationAngle = Me.mySensor.ElevationAngle - 5
        End If
    End Sub


    Private Sub butAbout_MouseEnter(sender As System.Object, e As System.Windows.Input.MouseEventArgs) Handles butAbout.MouseEnter
        Me.butAbout.Foreground = Brushes.Blue
    End Sub


    Private Sub butAbout_MouseLeave(sender As Object, e As System.Windows.Input.MouseEventArgs) Handles butAbout.MouseLeave
        Me.butAbout.Foreground = Brushes.Black
    End Sub

    ''' <summary>
    ''' Opens the about window
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub butAbout_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles butAbout.Click
        Dim aboutForm As New multipurpose
        aboutForm.panel2Visible()
        aboutForm.formTitle("About")
        aboutForm.Show()
    End Sub

    ''' <summary>
    ''' Ensures input entered in the form is numeric
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TextBox1_TextChanged(sender As System.Object, e As System.Windows.Controls.TextChangedEventArgs) Handles TextBox1.TextChanged
        If Not Microsoft.VisualBasic.IsNumeric(Me.TextBox1.Text) Then
            Me.TextBox1.Text = "16"
        End If
    End Sub

    ''' <summary>
    ''' Stop the live mode
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub stopLiveMode()
        Me.quitLiveModeStatus = True
        Me.TextBox1.IsReadOnly = False
        Me.mydriver.offLiveMode()
        Me.frameCounter = 0
        Me.labelDebug.Content = CStr(Me.frameCounter)
        Me.butLiveMode.Content = "Live Mode"
        Me.Button1.IsEnabled = True
    End Sub

    ''' <summary>
    ''' Toggling all button enable status
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub toggleAllButton()
        If Me.butDecreaseElevation.IsEnabled = True Then
            Me.butDecreaseElevation.IsEnabled = False
            Me.butIncreaseElevation.IsEnabled = False
            Me.butLiveMode.IsEnabled = False
            Me.Button1.IsEnabled = False
        Else
            Me.butDecreaseElevation.IsEnabled = True
            Me.butIncreaseElevation.IsEnabled = True
            Me.butLiveMode.IsEnabled = True
            Me.Button1.IsEnabled = True
        End If

    End Sub
End Class