Imports System
Imports System.Collections.Generic
Imports System.IO

''' <summary>
''' The class that will create the password vector and do the password and attempt comparison
''' </summary>
''' <remarks></remarks>
Public Class Password

    Private trainingSet As List(Of FeatureVector)
    Private thresholds As List(Of Double)
    Private xThresholds As List(Of Double)
    Private yThresholds As List(Of Double)
    Private zThresholds As List(Of Double)
    ' private NaiveBayesClassifier classifier;
    ' distance from head to hip center, we will store the average distance calculated from the entire training set. Somewhat reliable, but not fool proof
    Private usersHeight As Double
    Private usersHeightThreshold As Double
    Private myPassword As List(Of FeaturePoint) ' will be the average value of points from training set
    Private Shared ReadOnly featurePoints() As String = {"HipCenter", "Spine", "ShoulderCenter", "Head", "ShoulderLeft", "ElbowLeft", "WristLeft", "HandLeft", "ShoulderRight", "ElbowRight", "WristRight", "HandRight", "HipLeft", "KneeLeft", "AnkleLeft", "FootLeft", "HipRight", "KneeRight", "AnkleRight", "FootRight"}
    Private ReadOnly thresholdConstant As Double
    Private ReadOnly xthresholdConstant As Double
    Private ReadOnly ythresholdConstant As Double
    Private ReadOnly zthresholdConstant As Double
    Private mostRecentLogin As FeatureVector

    ''' <summary>
    ''' Constructor. Creates a password given a list of FeatureVectors
    ''' </summary>
    ''' <param name="train">the training set for the password</param>
    ''' <param name="constant"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal train As List(Of FeatureVector), ByVal constant As Double)
        Me.trainingSet = train ' save the training set
        Me.thresholdConstant = constant
        Me.myPassword = New List(Of FeaturePoint)()
        Me.setPassword()
        Me.thresholds = New List(Of Double)()
        Me.usersHeight = 0
        Me.usersHeightThreshold = 0
        Me.calculateThresholds()
        Me.setBiometrics()
        Me.zthresholdConstant = 0
        Me.ythresholdConstant = 0
        Me.xthresholdConstant = 0
    End Sub

    ''' <summary>
    ''' Constructor. Creates a password given a list of FeatureVectors and constants
    ''' </summary>
    ''' <param name="train"></param>
    ''' <param name="xConstant"></param>
    ''' <param name="yConstant"></param>
    ''' <param name="zConstant"></param>
    ''' <param name="heightThreshold"></param>
    ''' <remarks></remarks>
    Public Sub New(ByVal train As List(Of FeatureVector), ByVal xConstant As Double, ByVal yConstant As Double, ByVal zConstant As Double, ByVal heightThreshold As Double)
        Me.trainingSet = train ' save the training set
        Me.thresholdConstant = heightThreshold
        Me.xthresholdConstant = xConstant
        Me.ythresholdConstant = yConstant
        Me.zthresholdConstant = zConstant
        Me.myPassword = New List(Of FeaturePoint)()
        Me.setPassword()
        Me.thresholds = New List(Of Double)()
        Me.xThresholds = New List(Of Double)()
        Me.yThresholds = New List(Of Double)()
        Me.zThresholds = New List(Of Double)()
        Me.usersHeight = 0
        Me.usersHeightThreshold = 0
        Me.calculateIndividualXYZThresholds()
        Me.setBiometrics()
    End Sub

    ''' <summary>
    ''' Creates the password using the training set. The password consists of the average value for each feature point.
    ''' The password class variable is set after this method executes.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub setPassword()
        Dim p1 As FeaturePoint
        Dim x As Double = 0
        Dim y As Double = 0
        Dim z As Double = 0
        For i As Integer = 0 To Me.trainingSet(0).size() - 1
            x = 0
            y = 0
            z = 0
            For a As Integer = 0 To Me.trainingSet.Count - 1
                p1 = Me.trainingSet(a).getFeaturePoint(i)
                x += p1.xVal
                y += p1.yVal
                z += p1.zVal
            Next a
            x = x / Me.trainingSet.Count
            y = y / Me.trainingSet.Count
            z = z / Me.trainingSet.Count
            Dim average As New FeaturePoint(x, y, z)
            Me.myPassword.Add(average)
        Next i
    End Sub

    ''' <summary>
    ''' figures out the percentage that a login attempt matched the password
    ''' </summary>
    ''' <param name="numFailedToMatch">the number of feature points that failed to match the password</param>
    ''' <returns>the percent that the login attemp matched the password</returns>
    ''' <remarks></remarks>
    Public Overridable Function percentMatching(ByVal numFailedToMatch As Integer) As Double
        Dim m As Double = CDbl(Me.myPassword.Count - numFailedToMatch) / Me.myPassword.Count
        Return m * 100
    End Function

    ''' <summary>
    ''' This is a login attempt. The object is past some other FeatureVector, which it compares to the values
    ''' that it has stored as the password. The users height as well as each feature point is compared.
    ''' </summary>
    ''' <param name="f1">some other feature vector, presumably an attemped login</param>
    ''' <returns>the number of feature points that failed to match the password</returns>
    ''' <remarks></remarks>
    Public Overridable Function login(ByVal f1 As FeatureVector) As Integer
        Me.mostRecentLogin = f1 ' save this login
        ' get the height of person from f1 and compare to the height of the known user
        Dim height As Double = f1.getFeaturePoint("Head").distance(f1.getFeaturePoint("HipCenter"))
        If Me.distance(height, Me.usersHeight) > Me.usersHeightThreshold Then
            Console.WriteLine("height does not match")
        Else
            Console.WriteLine("height matches")
        End If
        Dim fp1 As List(Of FeaturePoint) = f1.featurePointsList
        Dim passwordFeaturePoints As List(Of FeaturePoint) = Me.myPassword
        Dim count As Integer = 0
        For i As Integer = 0 To fp1.Count - 1
            If fp1(i).distance(passwordFeaturePoints(i)) > Me.thresholds(i) Then
                Console.WriteLine("failed at point: " & featurePoints(i))
                Console.WriteLine("Distance: " & fp1(i).distance(passwordFeaturePoints(i)))
                count += 1
            End If
        Next i
        Return count
    End Function

    ''' <summary>
    ''' This is a login attempt. The object is past some other FeatureVector, which it compares to the values
    ''' that it has stored as the password. The users height as well as each feature point is compared.
    ''' </summary>
    ''' <param name="f1">some other feature vector, presumably an attempted login</param>
    ''' <param name="a">overloading dummy</param>
    ''' <returns>the number of feature points that failed to match the password</returns>
    ''' <remarks></remarks>
    Public Overridable Function login(ByVal f1 As FeatureVector, ByVal a As Integer) As Integer()
        Me.mostRecentLogin = f1 ' save this login
        ' get the height of person from f1 and compare to the height of the known user
        Dim height As Double = f1.getFeaturePoint("Head").distance(f1.getFeaturePoint("HipCenter"))
        If Me.distance(height, Me.usersHeight) > Me.usersHeightThreshold Then
            Console.WriteLine("height does not match")
        Else
            Console.WriteLine("height matches")
        End If
        Dim fp1 As List(Of FeaturePoint) = f1.featurePointsList
        Dim passwordFeaturePoints As List(Of FeaturePoint) = Me.myPassword
        Dim xCount As Integer = 0
        Dim yCount As Integer = 0
        Dim zCount As Integer = 0
        Dim countReturn(2) As Integer
        For i As Integer = 0 To fp1.Count - 1
            If Me.distance(fp1(i).xVal, passwordFeaturePoints(i).xVal) > Me.xThresholds(i) Then
                Console.WriteLine("failed in X component at point: " + featurePoints(i))
                Console.WriteLine("Distance: " + CStr(Me.distance(fp1(i).xVal, passwordFeaturePoints(i).xVal)))
                xCount += 1
            End If
            If Me.distance(fp1(i).yVal, passwordFeaturePoints(i).yVal) > Me.yThresholds(i) Then
                Console.WriteLine("failed in Y component at point: " + featurePoints(i))
                Console.WriteLine("Distance: " & Me.distance(fp1(i).yVal, passwordFeaturePoints(i).yVal))
                yCount += 1
            End If
            If Me.distance(fp1(i).zVal, passwordFeaturePoints(i).zVal) > Me.zThresholds(i) Then
                Console.WriteLine("failed in Z component at point: " & featurePoints(i))
                Console.WriteLine("Distance: " & Me.distance(fp1(i).zVal, passwordFeaturePoints(i).zVal))
                zCount += 1
            End If
        Next i
        countReturn(0) = xCount
        countReturn(1) = yCount
        countReturn(2) = zCount
        Return countReturn
    End Function

    ''' <summary>
    ''' calculates acceptable thresholds for each FeaturePoint using the training set. The acceptable threshold is the max distance
    ''' between the same feature point within the training set. Essentially we say that an acceptable threshold is the worst job
    ''' the user was able to in re-position their feature point in the same place. Note that this does not set the users height
    ''' or the users height threshold.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub calculateThresholds()
        Dim maxDistance As Double = -1
        Dim tempDistance As Double = -1
        Dim p1 As FeaturePoint
        Dim p2 As FeaturePoint
        For i As Integer = 0 To Me.trainingSet(0).size() - 1
            p1 = Me.trainingSet(0).getFeaturePoint(i) ' go through all points of 1st feature vector and compare with all other corresponding points in other FeatureVector
            For x As Integer = 1 To Me.trainingSet.Count - 1
                p2 = Me.trainingSet(x).getFeaturePoint(i)
                tempDistance = p1.distance(p2)
                If tempDistance > maxDistance Then
                    maxDistance = tempDistance
                End If
            Next x
            thresholds.Add(maxDistance + thresholdConstant)
            maxDistance = 0
        Next i
    End Sub

    ''' <summary>
    ''' calculates acceptable thresholds for each FeaturePoint using the training set. The acceptable threshold is the max distance
    ''' between the same feature point within the training set. Essentially we say that an acceptable threshold is the worst job
    ''' the user was able to in re-position their feature point in the same place. Note that this does not set the users height
    ''' or the users height threshold.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub calculateIndividualXYZThresholds()
        Dim x1 As Double = 0
        Dim x2 As Double = 0
        Dim y1 As Double = 0
        Dim y2 As Double = 0
        Dim z1 As Double = 0
        Dim z2 As Double = 0
        Dim maxXDistance As Double = -1
        Dim tempXDistance As Double = -1
        Dim maxYDistance As Double = -1
        Dim tempYDistance As Double = -1
        Dim maxZDistance As Double = -1
        Dim tempZDistance As Double = -1
        Dim p1 As FeaturePoint
        Dim p2 As FeaturePoint
        For i As Integer = 0 To Me.trainingSet(0).size() - 1
            p1 = Me.trainingSet(0).getFeaturePoint(i) ' go through all points of 1st feature vector and compare with all other corresponding points in other FeatureVector
            x1 = p1.xVal
            y1 = p1.yVal
            z1 = p1.zVal
            For x As Integer = 1 To Me.trainingSet.Count - 1
                p2 = Me.trainingSet(x).getFeaturePoint(i)
                x2 = p2.xVal
                y2 = p2.yVal
                z2 = p2.zVal
                tempXDistance = Me.distance(x1, x2)
                tempYDistance = Me.distance(y1, y2)
                tempZDistance = Me.distance(z1, z2)
                If tempXDistance > maxXDistance Then
                    maxXDistance = tempXDistance
                End If
                If tempYDistance > maxYDistance Then
                    maxYDistance = tempYDistance
                End If
                If tempZDistance > maxZDistance Then
                    maxZDistance = tempZDistance
                End If
            Next x
            Me.xThresholds.Add(maxXDistance + Me.xthresholdConstant)
            Me.yThresholds.Add(maxYDistance + Me.ythresholdConstant)
            Me.zThresholds.Add(maxZDistance + Me.zthresholdConstant)
            maxXDistance = 0
            maxYDistance = 0
            maxZDistance = 0
        Next i
    End Sub

    ''' <summary>
    ''' Each feature vector has a list of FeaturePoints that are meant to be biometrics => identifying characteristics. As of now we will be using
    ''' the persons height (distance from head to hip center). This method calculates the average distance from head to hip center based on the training
    ''' set as well as calculates an acceptable threshold for the users height. Again, we use the max distance between the users height that occured
    ''' in the training set as the threshold. Note that this sets the users height as well as the users height threshold
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub setBiometrics()
        Dim heights As New List(Of Double)()
        For x As Integer = 0 To Me.trainingSet.Count - 1
            heights.Add((Me.trainingSet(x).getFeaturePoint("Head").distance(Me.trainingSet(x).getFeaturePoint("HipCenter"))))
        Next x
        ' calculate the users average height using points from the training set
        For i As Integer = 0 To heights.Count - 1
            Me.usersHeight = CDbl(Me.usersHeight + heights(i))
        Next i
        Me.usersHeight = Me.usersHeight / heights.Count

        Dim height1 As Double = 0
        Dim height2 As Double = 0
        Dim tempHeightDistance As Double = 0
        Dim maxHeightDistance As Double = 0
        ' find threshold for height
        For i As Integer = 0 To heights.Count - 1
            height1 = CDbl(heights(i))
            For x As Integer = i + 1 To heights.Count - 1
                height2 = CDbl(heights(x))
                tempHeightDistance = Me.distance(height1, height2)
                If tempHeightDistance > maxHeightDistance Then
                    maxHeightDistance = tempHeightDistance
                End If
            Next x
        Next i
        Me.usersHeightThreshold = maxHeightDistance + thresholdConstant
    End Sub

    ''' <summary>
    ''' outputs useful information to a text file
    ''' </summary>
    ''' <param name="filename"></param>
    ''' <remarks></remarks>
    Public Overridable Sub writeToTextFile(ByVal filename As String)
        Try
            Dim out As New StreamWriter(filename)
            out.WriteLine("Thresholds for all feature points: ")
            For i As Integer = 0 To Me.thresholds.Count - 1
                out.WriteLine(featurePoints(i) + ": " + CStr(Me.thresholds(i)))
            Next i
            out.WriteLine()
            out.WriteLine("Threshold for users height: " + CStr(Me.usersHeightThreshold))
            out.WriteLine("Users height: " + CStr(Me.usersHeight))
            out.WriteLine()
            out.WriteLine("Users password (average of the training set): ")
            For i As Integer = 0 To Me.myPassword.Count - 1
                Dim p1 As FeaturePoint = Me.myPassword(i)
                out.WriteLine(CStr(p1.xVal) + " " + CStr(p1.yVal) + " " + CStr(p1.zVal))
            Next i
            out.Close()
        Catch e As FileNotFoundException
            Console.WriteLine(e.Message)
            Return
        End Try
        Return
    End Sub

    ''' <summary>
    ''' outputs useful information to the console
    ''' </summary>
    ''' <remarks></remarks>
    Public Overridable Sub output()
        Console.WriteLine("Thresholds for all feature points: ")
        For i As Integer = 0 To Me.thresholds.Count - 1
            Console.WriteLine(featurePoints(i) + ": " + CStr(Me.thresholds(i)))
        Next i
        Console.WriteLine()
        Console.WriteLine("Threshold for users height: " + CStr(Me.usersHeightThreshold))
        Console.WriteLine("Users height: " + CStr(Me.usersHeight))
        Dim height As Double = Me.mostRecentLogin.getFeaturePoint("Head").distance(Me.mostRecentLogin.getFeaturePoint("HipCenter"))
        Console.WriteLine("Height of previous person to login: " + CStr(height))
    End Sub

    ''' <summary>
    ''' outputs useful information to the console
    ''' </summary>
    ''' <param name="index">overloading dummy</param>
    ''' <remarks></remarks>
    Public Overridable Sub output(ByVal index As Integer)
        Console.WriteLine("Thresholds for all feature points: ")
        For i As Integer = 0 To Me.xThresholds.Count - 1
            Console.WriteLine(featurePoints(i) + " X: " + CStr(Me.xThresholds(i)) + " Y: " + CStr(Me.yThresholds(i)) + " Z: " + CStr(Me.zThresholds(i)))
        Next i
        Console.WriteLine()
        Console.WriteLine("Threshold for users height: " + CStr(Me.usersHeightThreshold))
        Console.WriteLine("Users height: " & Me.usersHeight)
        Dim height As Double = Me.mostRecentLogin.getFeaturePoint("Head").distance(Me.mostRecentLogin.getFeaturePoint("HipCenter"))
        Console.WriteLine("Height of previous person to login: " & height)
    End Sub

    ''' <summary>
    ''' Calculates 1 dimensional distance between 2 numbers
    ''' </summary>
    ''' <param name="x1"></param>
    ''' <param name="x2"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function distance(ByVal x1 As Double, ByVal x2 As Double) As Double
        Return (Math.Abs(x1 - x2))
    End Function
End Class
