Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Text.RegularExpressions

''' <summary>
''' The class that will handle the comparison algorithm
''' </summary>
''' <remarks></remarks>
Public Class driver
    Private myFvFolder As String
    Private myAttempt As String
    Private liveModeStat As Boolean = False
    Private numFiles As Integer = 16

    ''' <summary>
    ''' Initialize the needed parameters for the login method
    ''' </summary>
    ''' <param name="fvFolderName"></param>
    ''' <param name="attemptName"></param>
    ''' <remarks></remarks>
    Public Sub mainDriver(ByVal fvFolderName As String, ByVal attemptName As String)
        Me.myFvFolder = fvFolderName
        Me.myAttempt = attemptName
        run()

    End Sub

    ''' <summary>
    ''' Run the comparison algorithm with the provided parameters
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub run()
        Dim myRegex As New Regex("\.txt$")
        Dim FV1 As FeatureVector
        Dim trainingSet As New List(Of FeatureVector)()
        Try
            For i As Integer = 1 To numFiles - 1
                FV1 = New FeatureVector(myFvFolder & "fv" & i & ".txt")
                trainingSet.Add(FV1)
            Next i
            Dim p1 As New Password(trainingSet, 200, 80, 300, 0) '30)
            Try
                If myRegex.Matches(myAttempt).Count > 0 Then
                    FV1 = New FeatureVector(myAttempt)
                Else
                    FV1 = New FeatureVector(myAttempt & "attempt.txt")
                End If
            Catch ex As Exception
                Return
            End Try
            
            Console.WriteLine("Loaded from dataset: " & myFvFolder)
            Console.WriteLine("Loaded attempt file from: " & myAttempt)

            Dim counts() As Integer = p1.login(FV1, 10)
            Console.WriteLine("Number of X points that failed to match: " & counts(0))
            Console.WriteLine("Percent match in X: " & p1.percentMatching(counts(0)))
            Console.WriteLine("Number of Y points that failed to match: " & counts(1))
            Console.WriteLine("Percent match in Y: " & p1.percentMatching(counts(1)))
            Console.WriteLine("Number of Z points that failed to match: " & counts(2))
            Console.WriteLine("Percent match in Z: " & p1.percentMatching(counts(2)))
            p1.writeToTextFile(myFvFolder & "/output.txt")
            p1.output(1)
            If counts(0) = 0 And counts(1) = 0 And counts(2) = 0 Then
                MessageBox.Show("Matched")
            Else
                If liveModeStat = False Then
                    MessageBox.Show("No match")
                End If
            End If
            Console.WriteLine()
            trainingSet = New List(Of FeatureVector)()
            GC.Collect()
        Catch e As Exception
            MessageBox.Show(e.Message)
        End Try
    End Sub

    Public Sub offLiveMode()
        Me.liveModeStat = False
    End Sub

    Public Sub onLiveMode()
        Me.liveModeStat = True
    End Sub

    Public Sub setNumFiles(ByVal num As Integer)
        Me.numFiles = num
    End Sub
End Class
